GDK_PIXBUF = {
    'debian': 'gir1.2-gdkpixbuf-2.0',
    'linuxmint': 'gir1.2-gdkpixbuf-2.0',
    'raspbian': 'gir1.2-gdkpixbuf-2.0',
    'ubuntu': 'gir1.2-gdkpixbuf-2.0',
}

GTK = {
    'debian': 'gir1.2-gtk-3.0',
    'fedora': 'gtk3',
    'gentoo': 'x11-libs/gtk+',
    'linuxmint': 'gir1.2-gtk-3.0',
    'raspbian': 'gir1.2-gtk-3.0',
    'suse': 'python-gtk',
    'ubuntu': 'gir1.2-gtk-3.0',
}

NOTIFY = {
    'debian': 'gir1.2-notify-0.7',
    'fedora': 'libnotify',
    'ubuntu': 'gir1.2-notify-0.7',
}

PANGO = {
    'debian': 'gir1.2-pango-1.0',
    'linuxmint': 'gir1.2-pango-1.0',
    'raspbian': 'gir1.2-pango-1.0',
    'ubuntu': 'gir1.2-pango-1.0',
}
